{% load sekizai_tags %}

{% render_block "one" %}
{% render_block "two" %}

{% addtoblock "one" %}
the same file
{% endaddtoblock %}

{% addtoblock "two" %}
the same file
{% endaddtoblock %}

{% addtoblock "one" %}
the same file
{% endaddtoblock %}